@@ -90,17 +90,37 @@ static int __get_drm_device_name(int fd, char *name)
 	return -1;
 }
 
-static bool is_i915_device(int fd)
+static bool __is_device(int fd, const char *expect)
 {
-	int ret;
 	char name[5] = "";
 
-	ret = __get_drm_device_name(fd, name);
+	if (__get_drm_device_name(fd, name))
+		return false;
 
-	return !ret && strcmp("i915", name) == 0;
+	return strcmp(expect, name) == 0;
 }
 
-static bool is_intel(int fd)
+bool is_i915_device(int fd)
+{
+	return __is_device(fd, "i915");
+}
+
+static bool is_vc4_device(int fd)
+{
+	return __is_device(fd, "vc4");
+}
+
+static bool is_vgem_device(int fd)
+{
+	return __is_device(fd, "vgem");
+}
+
+static bool is_virtio_device(int fd)
+{
+	return __is_device(fd, "virt");
+}
+
+static bool has_known_intel_chipset(int fd)
 {
 	struct drm_i915_getparam gp;
 	int devid = 0;
@@ -112,25 +132,13 @@ static bool is_intel(int fd)
 	if (ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp)))
 		return false;
 
-	if (!IS_INTEL(devid))
+	if (!intel_gen(devid))
 		return false;
 
 	__drm_device_id = devid;
 	return true;
 }
 
-static void check_stop_rings(void)
-{
-	enum stop_ring_flags flags;
-	flags = igt_get_stop_rings();
-	igt_warn_on_f(flags != 0,
-		      "i915_ring_stop flags on exit 0x%x, can't quiescent gpu cleanly\n",
-		      flags);
-
-	if (flags)
-		igt_set_stop_rings(STOP_RING_NONE);
-}
-
 #define LOCAL_I915_EXEC_VEBOX	(4 << 0)
 /**
  * gem_quiescent_gpu:
@@ -146,41 +154,33 @@ static void check_stop_rings(void)
  */
 void gem_quiescent_gpu(int fd)
 {
-	uint32_t batch[2] = {MI_BATCH_BUFFER_END, 0};
+	uint32_t bbe = MI_BATCH_BUFFER_END;
 	struct drm_i915_gem_execbuffer2 execbuf;
-	struct drm_i915_gem_exec_object2 gem_exec[1];
+	struct drm_i915_gem_exec_object2 obj;
+	unsigned ring;
 
-	check_stop_rings();
-
-	memset(gem_exec, 0, sizeof(gem_exec));
-	gem_exec[0].handle = gem_create(fd, 4096);
-	gem_write(fd, gem_exec[0].handle, 0, batch, sizeof(batch));
+	memset(&obj, 0, sizeof(obj));
+	obj.handle = gem_create(fd, 4096);
+	gem_write(fd, obj.handle, 0, &bbe, sizeof(&bbe));
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)gem_exec;
+	execbuf.buffers_ptr = (uintptr_t)&obj;
 	execbuf.buffer_count = 1;
 
-	execbuf.flags = I915_EXEC_RENDER;
-	gem_execbuf(fd, &execbuf);
-
-	if (gem_has_blt(fd)) {
-		execbuf.flags = I915_EXEC_BLT;
-		gem_execbuf(fd, &execbuf);
+	for (ring = 0; ring < 1<<6; ring++) {
+		execbuf.flags = ring;
+		__gem_execbuf(fd, &execbuf);
 	}
 
-	if (gem_has_bsd(fd)) {
-		execbuf.flags = I915_EXEC_BSD;
-		gem_execbuf(fd, &execbuf);
+	if (gem_has_bsd2(fd)) {
+		execbuf.flags = I915_EXEC_BSD | (2 << 13);
+		__gem_execbuf(fd, &execbuf);
 	}
 
-	if (gem_has_vebox(fd)) {
-		execbuf.flags = LOCAL_I915_EXEC_VEBOX;
-		gem_execbuf(fd, &execbuf);
-	}
+	gem_sync(fd, obj.handle);
+	gem_close(fd, obj.handle);
 
-	gem_sync(fd, gem_exec[0].handle);
 	igt_drop_caches_set(DROP_RETIRE);
-	gem_close(fd, gem_exec[0].handle);
 }
 
 /**
@@ -209,7 +209,7 @@ int drm_get_card(void)
 		if (fd == -1)
 			continue;
 
-		if (!is_i915_device(fd) || !is_intel(fd)) {
+		if (!is_i915_device(fd) || !has_known_intel_chipset(fd)) {
 			close(fd);
 			continue;
 		}
@@ -223,6 +223,13 @@ int drm_get_card(void)
 	return -1;
 }
 
+static int modprobe(const char *driver)
+{
+	char buf[128];
+	snprintf(buf, sizeof(buf), "/sbin/modprobe -s %s", driver);
+	return system(buf);
+}
+
 /**
  * __drm_open_driver:
  * @chipset: OR'd flags for each chipset to search, eg. #DRIVER_INTEL
@@ -234,25 +241,37 @@ int drm_get_card(void)
  */
 int __drm_open_driver(int chipset)
 {
+	if (chipset & DRIVER_VGEM)
+		modprobe("vgem");
+
 	for (int i = 0; i < 16; i++) {
 		char name[80];
 		int fd;
-		bool found_intel;
 
 		sprintf(name, "/dev/dri/card%u", i);
 		fd = open(name, O_RDWR);
 		if (fd == -1)
 			continue;
 
-		found_intel =  is_i915_device(fd) && is_intel(fd) && (chipset & DRIVER_INTEL);
+		if (chipset & DRIVER_INTEL && is_i915_device(fd) &&
+		    has_known_intel_chipset(fd))
+			return fd;
 
-		if ((chipset & DRIVER_ANY) || found_intel)
+		if (chipset & DRIVER_VC4 &&
+		    is_vc4_device(fd))
+			return fd;
+
+		if (chipset & DRIVER_VGEM &&
+		    is_vgem_device(fd))
+			return fd;
+
+		if (chipset & DRIVER_VIRTIO &&
+		    is_virtio_device(fd))
 			return fd;
 
 		close(fd);
 	}
 
-	igt_skip("No intel gpu found\n");
 	return -1;
 }
 
@@ -273,7 +292,7 @@ static int __drm_open_driver_render(int chipset)
 		if (fd == -1)
 			continue;
 
-		if (!is_i915_device(fd) || !is_intel(fd)) {
+		if (!is_i915_device(fd) || !has_known_intel_chipset(fd)) {
 			close(fd);
 			fd = -1;
 			continue;
@@ -293,7 +312,6 @@ static void quiescent_gpu_at_exit(int sig)
 	if (at_exit_drm_fd < 0)
 		return;
 
-	check_stop_rings();
 	gem_quiescent_gpu(at_exit_drm_fd);
 	close(at_exit_drm_fd);
 	at_exit_drm_fd = -1;
@@ -304,7 +322,6 @@ static void quiescent_gpu_at_exit_render(int sig)
 	if (at_exit_drm_render_fd < 0)
 		return;
 
-	check_stop_rings();
 	gem_quiescent_gpu(at_exit_drm_render_fd);
 	close(at_exit_drm_render_fd);
 	at_exit_drm_render_fd = -1;
@@ -322,14 +339,15 @@ static void quiescent_gpu_at_exit_render(int sig)
 int drm_open_driver(int chipset)
 {
 	static int open_count;
-	int fd = __drm_open_driver(chipset);
+	int fd;
 
-	igt_require(fd >= 0);
+	fd = __drm_open_driver(chipset);
+	igt_skip_on_f(fd<0, "No known gpu found\n");
 
 	if (__sync_fetch_and_add(&open_count, 1))
 		return fd;
 
-   if(chipset & DRIVER_INTEL){
+	if (is_i915_device(fd)) {
 		gem_quiescent_gpu(fd);
 		igt_install_exit_handler(quiescent_gpu_at_exit);
 	}
@@ -351,7 +369,6 @@ int drm_open_driver_master(int chipset)
 {
 	int fd = drm_open_driver(chipset);
 
-	igt_require(fd >= 0);
 	igt_require_f(drmSetMaster(fd) == 0, "Can't become DRM master, "
 		      "please check if no other DRM client is running.\n");
 
@@ -387,3 +404,8 @@ int drm_open_driver_render(int chipset)
 
 	return fd;
 }
+
+void igt_require_intel(int fd)
+{
+	igt_require(is_i915_device(fd) && has_known_intel_chipset(fd));
+}
