@@ -56,7 +56,7 @@
 #include <ctype.h>
 #include <limits.h>
 #include <locale.h>
-#include <fnmatch.h>
+#include <uwildmat/uwildmat.h>
 
 #include "drmtest.h"
 #include "intel_chipset.h"
@@ -97,7 +97,9 @@
  *
  * To allow this i-g-t provides #igt_fixture code blocks for setup code outside
  * of subtests and automatically skips the subtest code blocks themselves. For
- * special cases igt_only_list_subtests() is also provided.
+ * special cases igt_only_list_subtests() is also provided. For setup code only
+ * shared by a group of subtest encapsulate the #igt_fixture block and all the
+ * subtestest in a #igt_subtest_group block.
  *
  * # Magic Control Blocks
  *
@@ -209,6 +211,19 @@
  * intel gpu to be present). Then individual subtests can be run with
  * "--run-subtest". Usage help for tests with subtests can be obtained with the
  * "--help" command line option.
+ *
+ * A wildcard expression can be given to --run-subtest to specify a subset of
+ * subtests to run. See https://tools.ietf.org/html/rfc3977#section-4 for a
+ * description of allowed wildcard expressions.
+ * Some examples of allowed wildcard expressions are:
+ *
+ * - '*basic*' match any subtest containing basic
+ * - 'basic-???' match any subtest named basic- with 3 characters after -
+ * - 'basic-[0-9]' match any subtest named basic- with a single number after -
+ * - 'basic-[^0-9]' match any subtest named basic- with a single non numerical character after -
+ * - 'basic*,advanced*' match any subtest starting basic or advanced
+ * - '*,!basic*' match any subtest not starting basic
+ * - 'basic*,!basic-render*' match any subtest starting basic but not starting basic-render
  */
 
 static unsigned int exit_handler_count;
@@ -383,6 +398,23 @@ error:
 	return -errno;
 }
 
+#define MSEC_PER_SEC (1000)
+#define USEC_PER_SEC (1000*MSEC_PER_SEC)
+#define NSEC_PER_SEC (1000*USEC_PER_SEC)
+uint64_t igt_nsec_elapsed(struct timespec *start)
+{
+	struct timespec now;
+
+	gettime(&now);
+	if ((start->tv_sec | start->tv_nsec) == 0) {
+		*start = now;
+		return 0;
+	}
+
+	return ((now.tv_nsec - start->tv_nsec) +
+		(uint64_t)NSEC_PER_SEC*(now.tv_sec - start->tv_sec));
+}
+
 bool __igt_fixture(void)
 {
 	assert(!in_fixture);
@@ -540,7 +572,7 @@ static void oom_adjust_for_doom(void)
 
 static int common_init(int *argc, char **argv,
 		       const char *extra_short_opts,
-		       struct option *extra_long_opts,
+		       const struct option *extra_long_opts,
 		       const char *help_str,
 		       igt_opt_handler_t extra_opt_handler,
 		       void *handler_data)
@@ -746,7 +778,7 @@ out:
  */
 int igt_subtest_init_parse_opts(int *argc, char **argv,
 				const char *extra_short_opts,
-				struct option *extra_long_opts,
+				const struct option *extra_long_opts,
 				const char *help_str,
 				igt_opt_handler_t extra_opt_handler,
 				void *handler_data)
@@ -777,7 +809,7 @@ enum igt_log_level igt_log_level = IGT_LOG_INFO;
  */
 void igt_simple_init_parse_opts(int *argc, char **argv,
 				const char *extra_short_opts,
-				struct option *extra_long_opts,
+				const struct option *extra_long_opts,
 				const char *help_str,
 				igt_opt_handler_t extra_opt_handler,
 				void *handler_data)
@@ -814,7 +846,7 @@ bool __igt_run_subtest(const char *subtest_name)
 	}
 
 	if (run_single_subtest) {
-		if (fnmatch(run_single_subtest, subtest_name, 0) != 0)
+		if (uwildmat(subtest_name, run_single_subtest) == 0)
 			return false;
 		else
 			run_single_subtest_found = true;
@@ -859,6 +891,16 @@ bool igt_only_list_subtests(void)
 	return list_subtests;
 }
 
+void __igt_subtest_group_save(int *save)
+{
+	*save = skip_subtests_henceforth;
+}
+
+void __igt_subtest_group_restore(int save)
+{
+	skip_subtests_henceforth = save;
+}
+
 static bool skipped_one = false;
 static bool succeeded_one = false;
 static bool failed_one = false;
@@ -1056,6 +1098,179 @@ static void print_backtrace(void)
 		       (unsigned int) off);
 	}
 }
+
+static const char hex[] = "0123456789abcdef";
+
+static void
+xputch(int c)
+{
+	write(STDERR_FILENO, (const void *) &c, 1);
+}
+
+static int
+xpow(int base, int pow)
+{
+	int i, r = 1;
+
+	for (i = 0; i < pow; i++)
+		r *= base;
+
+	return r;
+}
+
+static void
+printnum(unsigned long long num, unsigned base)
+{
+	int i = 0;
+	unsigned long long __num;
+
+	/* determine from where we should start dividing */
+	do {
+		__num /= base;
+		i++;
+	} while (__num);
+
+	while (i--)
+		xputch(hex[num / xpow(base, i) % base]);
+}
+
+static size_t
+xstrlcpy(char *dst, const char *src, size_t size)
+{
+	char *dst_in;
+
+	dst_in = dst;
+	if (size > 0) {
+		while (--size > 0 && *src != '\0')
+			*dst++ = *src++;
+		*dst = '\0';
+	}
+
+	return dst - dst_in;
+}
+
+static void
+xprintfmt(const char *fmt, va_list ap)
+{
+	const char *p;
+	int ch, base;
+	unsigned long long num;
+
+	while (1) {
+		while ((ch = *(unsigned char *) fmt++) != '%') {
+			if (ch == '\0') {
+				return;
+			}
+			xputch(ch);
+		}
+
+		ch = *(unsigned char *) fmt++;
+		switch (ch) {
+		/* character */
+		case 'c':
+			xputch(va_arg(ap, int));
+			break;
+		/* string */
+		case 's':
+			if ((p = va_arg(ap, char *)) == NULL) {
+				p = "(null)";
+			}
+
+			for (; (ch = *p++) != '\0';) {
+				if (ch < ' ' || ch > '~') {
+					xputch('?');
+				} else {
+					xputch(ch);
+				}
+			}
+			break;
+		/* (signed) decimal */
+		case 'd':
+			num = va_arg(ap, int);
+			if ((long long) num < 0) {
+				xputch('-');
+				num = -(long long) num;
+			}
+			base = 10;
+			goto number;
+		/* unsigned decimal */
+		case 'u':
+			num = va_arg(ap, unsigned int);
+			base = 10;
+			goto number;
+		/* (unsigned) hexadecimal */
+		case 'x':
+			num = va_arg(ap, unsigned int);
+			base = 16;
+number:
+			printnum(num, base);
+			break;
+
+		/* The following are not implemented */
+
+		/* width field */
+		case '1': case '2':
+		case '3': case '4':
+		case '5': case '6':
+		case '7': case '8':
+		case '9':
+		case '.': case '#':
+		/* long */
+		case 'l':
+		/* octal */
+		case 'o':
+		/* pointer */
+		case 'p':
+		/* float */
+		case 'f':
+			abort();
+		/* escaped '%' character */
+		case '%':
+			xputch(ch);
+			break;
+		/* unrecognized escape sequence - just print it literally */
+		default:
+			xputch('%');
+			for (fmt--; fmt[-1] != '%'; fmt--)
+				; /* do nothing */
+			break;
+		}
+	}
+}
+
+/* async-safe printf */
+static void
+xprintf(const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+	xprintfmt(fmt, ap);
+	va_end(ap);
+}
+
+static void print_backtrace_sig_safe(void)
+{
+	unw_cursor_t cursor;
+	unw_context_t uc;
+	int stack_num = 0;
+
+	write(STDERR_FILENO, "Stack trace: \n", 15);
+
+	unw_getcontext(&uc);
+	unw_init_local(&cursor, &uc);
+	while (unw_step(&cursor) > 0) {
+		char name[255];
+		unw_word_t off;
+
+		if (unw_get_proc_name(&cursor, name, 255, &off) < 0)
+			xstrlcpy(name, "<unknown>", 9);
+
+		xprintf(" #%d [%s+0x%x]\n", stack_num++, name,
+				(unsigned int) off);
+
+	}
+}
 #endif
 
 void __igt_fail_assert(const char *domain, const char *file, const int line,
@@ -1111,13 +1326,30 @@ void igt_exit(void)
 		exit(IGT_EXIT_INVALID);
 	}
 
-
 	if (igt_only_list_subtests())
 		exit(IGT_EXIT_SUCCESS);
 
-	kmsg(KERN_INFO "%s: exiting, ret=%d\n", command_str, igt_exitcode);
+	/* Calling this without calling one of the above is a failure */
+	assert(!test_with_subtests ||
+	       skipped_one ||
+	       succeeded_one ||
+	       failed_one);
+
+	if (test_with_subtests && !failed_one) {
+		if (succeeded_one)
+			igt_exitcode = IGT_EXIT_SUCCESS;
+		else
+			igt_exitcode = IGT_EXIT_SKIP;
+	}
+
+	if (command_str)
+		kmsg(KERN_INFO "%s: exiting, ret=%d\n",
+		     command_str, igt_exitcode);
 	igt_debug("Exiting with status code %d\n", igt_exitcode);
 
+	for (int c = 0; c < num_test_children; c++)
+		kill(test_children[c], SIGKILL);
+
 	if (!test_with_subtests) {
 		struct timespec now;
 		const char *result;
@@ -1138,19 +1370,11 @@ void igt_exit(void)
 				result = "FAIL";
 		}
 
-		printf("%s (%.3fs)\n", result, time_elapsed(&subtest_time, &now));
-		exit(igt_exitcode);
+		printf("%s (%.3fs)\n",
+		       result, time_elapsed(&subtest_time, &now));
 	}
 
-	/* Calling this without calling one of the above is a failure */
-	assert(skipped_one || succeeded_one || failed_one);
-
-	if (failed_one)
-		exit(igt_exitcode);
-	else if (succeeded_one)
-		exit(IGT_EXIT_SUCCESS);
-	else
-		exit(IGT_EXIT_SKIP);
+	exit(igt_exitcode);
 }
 
 /* fork support code */
@@ -1260,6 +1484,13 @@ int igt_wait_helper(struct igt_helper_process *proc)
 	return status;
 }
 
+static bool helper_was_alive(struct igt_helper_process *proc,
+			     int status)
+{
+	return (WIFSIGNALED(status) &&
+		WTERMSIG(status) == (proc->use_SIGKILL ? SIGKILL : SIGTERM));
+}
+
 /**
  * igt_stop_helper:
  * @proc: #igt_helper_process structure
@@ -1275,8 +1506,9 @@ void igt_stop_helper(struct igt_helper_process *proc)
 	kill(proc->pid, proc->use_SIGKILL ? SIGKILL : SIGTERM);
 
 	status = igt_wait_helper(proc);
-	assert(WIFSIGNALED(status) &&
-	       WTERMSIG(status) == (proc->use_SIGKILL ? SIGKILL : SIGTERM));
+	if (!helper_was_alive(proc, status))
+		igt_debug("Helper died too early with status=%d\n", status);
+	assert(helper_was_alive(proc, status));
 }
 
 static void children_exit_handler(int sig)
@@ -1388,6 +1620,22 @@ void igt_waitchildren(void)
 		igt_fail(err);
 }
 
+/**
+ * igt_waitchildren_timeout:
+ * @seconds: timeout in seconds to wait
+ * @reason: debug string explaining what timedout
+ *
+ * Wait for all children forked with igt_fork, for a maximum of @seconds.
+ *
+ * Wraps igt_waitchildren() and igt_set_timeout()
+ */
+void igt_waitchildren_timeout(int seconds, const char *reason)
+{
+	igt_set_timeout(seconds, reason);
+	igt_waitchildren();
+	igt_reset_timeout();
+}
+
 /* exit handler code */
 #define MAX_SIGNALS		32
 #define MAX_EXIT_HANDLERS	10
@@ -1403,12 +1651,12 @@ static struct {
 
 static igt_exit_handler_t exit_handler_fn[MAX_EXIT_HANDLERS];
 static bool exit_handler_disabled;
-static sigset_t saved_sig_mask;
 #define SIGDEF(x) { x, #x, sizeof(#x) - 1 }
 #define SILENT(x) { x, NULL, 0 }
 static const struct { int number; const char *name; size_t name_len; } handled_signals[] =
 	{ SILENT(SIGINT), SILENT(SIGHUP), SILENT(SIGTERM), SILENT(SIGQUIT),
-	  SILENT(SIGPIPE), SIGDEF(SIGABRT), SIGDEF(SIGSEGV), SIGDEF(SIGBUS) };
+	  SILENT(SIGPIPE), SIGDEF(SIGABRT), SIGDEF(SIGSEGV), SIGDEF(SIGBUS),
+	  SIGDEF(SIGFPE) };
 #undef SILENT
 #undef SIGDEF
 
@@ -1468,6 +1716,7 @@ static bool crash_signal(int sig)
 	switch (sig) {
 	case SIGILL:
 	case SIGBUS:
+	case SIGFPE:
 	case SIGSEGV:
 		return true;
 	default:
@@ -1497,7 +1746,9 @@ static void fatal_sig_handler(int sig)
 				igt_exitcode = 128 + sig;
 
 			failed_one = true;
-
+#ifdef HAVE_LIBUNWIND
+			print_backtrace_sig_safe();
+#endif
 			exit_subtest("CRASH");
 		}
 		break;
@@ -1577,51 +1828,6 @@ err:
 	igt_assert_f(0, "failed to install the signal handler\n");
 }
 
-/**
- * igt_disable_exit_handler:
- *
- * Temporarily disable all exit handlers. Useful for library code doing tricky
- * things.
- */
-void igt_disable_exit_handler(void)
-{
-	sigset_t set;
-	int i;
-
-	if (exit_handler_disabled)
-		return;
-
-	sigemptyset(&set);
-	for (i = 0; i < ARRAY_SIZE(handled_signals); i++)
-		sigaddset(&set, handled_signals[i].number);
-
-	if (sigprocmask(SIG_BLOCK, &set, &saved_sig_mask)) {
-		perror("sigprocmask");
-		return;
-	}
-
-	exit_handler_disabled = true;
-}
-
-/**
- * igt_enable_exit_handler:
- *
- * Re-enable all exit handlers temporarily disabled with
- * igt_disable_exit_handler().
- */
-void igt_enable_exit_handler(void)
-{
-	if (!exit_handler_disabled)
-		return;
-
-	if (sigprocmask(SIG_SETMASK, &saved_sig_mask, NULL)) {
-		perror("sigprocmask");
-		return;
-	}
-
-	exit_handler_disabled = false;
-}
-
 /* simulation enviroment support */
 
 /**
@@ -1748,7 +1954,7 @@ void igt_vlog(const char *domain, enum igt_log_level level, const char *format,
 		goto out;
 	}
 
-	line_continuation = line[strlen(line)] != '\n';
+	line_continuation = line[strlen(line) - 1] != '\n';
 
 	/* append log buffer */
 	_igt_log_buffer_append(formatted_line);
@@ -1855,6 +2061,10 @@ FILE *__igt_fopen_data(const char* igt_srcdir, const char* igt_datadir,
 		snprintf(path, sizeof(path), "%s/%s", igt_srcdir, filename);
 		fp = fopen(path, "r");
 	}
+	if (!fp) {
+		snprintf(path, sizeof(path), "./%s", filename);
+		fp = fopen(path, "r");
+	}
 
 	if (!fp)
 		igt_critical("Could not open data file \"%s\": %s", filename,
