@@ -46,6 +46,7 @@
 #elif defined(HAVE_SWAPCTL) /* Solaris */
 #include <sys/swap.h>
 #endif
+#include <sys/resource.h>
 
 #include "intel_io.h"
 #include "drmtest.h"
@@ -64,11 +65,8 @@ intel_get_total_ram_mb(void)
 
 #ifdef HAVE_STRUCT_SYSINFO_TOTALRAM /* Linux */
 	struct sysinfo sysinf;
-	int ret;
-
-	ret = sysinfo(&sysinf);
-	igt_assert(ret == 0);
 
+	igt_assert(sysinfo(&sysinf) == 0);
 	retval = sysinf.totalram;
 	retval *= sysinf.mem_unit;
 #elif defined(_SC_PAGESIZE) && defined(_SC_PHYS_PAGES) /* Solaris */
@@ -98,18 +96,10 @@ intel_get_avail_ram_mb(void)
 
 #ifdef HAVE_STRUCT_SYSINFO_TOTALRAM /* Linux */
 	struct sysinfo sysinf;
-	int fd, ret;
 
-	fd = open("/proc/sys/vm/drop_caches", O_RDWR);
-	if (fd != -1) {
-		ret = write(fd, "3\n", 2);
-		close(fd);
-		(void)ret;
-	}
-
-	ret = sysinfo(&sysinf);
-	igt_assert(ret == 0);
+	intel_purge_vm_caches();
 
+	igt_assert(sysinfo(&sysinf) == 0);
 	retval = sysinf.freeram;
 	retval *= sysinf.mem_unit;
 #elif defined(_SC_PAGESIZE) && defined(_SC_AVPHYS_PAGES) /* Solaris */
@@ -139,11 +129,8 @@ intel_get_total_swap_mb(void)
 
 #ifdef HAVE_STRUCT_SYSINFO_TOTALRAM /* Linux */
 	struct sysinfo sysinf;
-	int ret;
-
-	ret = sysinfo(&sysinf);
-	igt_assert(ret == 0);
 
+	igt_assert(sysinfo(&sysinf) == 0);
 	retval = sysinf.freeswap;
 	retval *= sysinf.mem_unit;
 #elif defined(HAVE_SWAPCTL) /* Solaris */
@@ -192,11 +179,83 @@ intel_get_total_swap_mb(void)
 	return retval / (1024*1024);
 }
 
+static uint64_t vfs_file_max(void)
+{
+	static long long unsigned max;
+	if (max == 0) {
+		FILE *file = fopen("/proc/sys/fs/file-max", "r");
+		max = 80000;
+		if (file) {
+			igt_assert(fscanf(file, "%llu", &max) == 1);
+			fclose(file);
+		}
+	}
+	return max;
+}
+
+static unsigned max_open_files(void)
+{
+	struct rlimit rlim;
+
+	if (getrlimit(RLIMIT_NOFILE, &rlim))
+		rlim.rlim_cur = 64 << 10;
+
+	return rlim.rlim_cur;
+}
+
+/**
+ * intel_require_files:
+ * @count: number of files that will be created
+ *
+ * Does the system support enough file descriptors for the test?
+ */
+void intel_require_files(uint64_t count)
+{
+	igt_require_f(count < max_open_files(),
+		      "Estimated that we need %'llu files, but the process maximum is only %'llu\n",
+		      (long long)count, (long long)max_open_files());
+}
+
+int __intel_check_memory(uint64_t count, uint64_t size, unsigned mode,
+			 uint64_t *out_required, uint64_t *out_total)
+{
+/* rough estimate of how many bytes the kernel requires to track each object */
+#define KERNEL_BO_OVERHEAD 512
+	uint64_t required, total;
+
+	required = count;
+	required *= size + KERNEL_BO_OVERHEAD;
+	required = ALIGN(required, 4096);
+
+	igt_debug("Checking %'llu surfaces of size %'llu bytes (total %'llu) against %s%s\n",
+		  (long long)count, (long long)size, (long long)required,
+		  mode & (CHECK_RAM | CHECK_SWAP) ? "RAM" : "",
+		  mode & CHECK_SWAP ? " + swap": "");
+
+	total = 0;
+	if (mode & (CHECK_RAM | CHECK_SWAP))
+		total += intel_get_avail_ram_mb();
+	if (mode & CHECK_SWAP)
+		total += intel_get_total_swap_mb();
+	total *= 1024 * 1024;
+
+	if (out_required)
+		*out_required = required;
+
+	if (out_total)
+		*out_total = total;
+
+	if (count > vfs_file_max())
+		return false;
+
+	return required < total;
+}
+
 /**
  * intel_require_memory:
  * @count: number of surfaces that will be created
  * @size: the size in bytes of each surface
- * @mode: a bitfield declaring whether the test will be run in RAM or in SWAP
+ * @mode: a bit field declaring whether the test will be run in RAM or in SWAP
  *
  * Computes the total amount of memory required to allocate @count surfaces,
  * each of @size bytes, and includes an estimate for kernel overhead. It then
@@ -210,52 +269,41 @@ intel_get_total_swap_mb(void)
  * there is not enough RAM + SWAP!
  *
  * If there is not enough RAM this function calls igt_skip with an appropriate
- * message. It only ever returns if the requirement is fullfilled. This function
+ * message. It only ever returns if the requirement is fulfilled. This function
  * also causes the test to be skipped automatically on simulation under the
  * assumption that any test that needs to check for memory requirements is a
  * thrashing test unsuitable for slow simulated systems.
  */
-void intel_require_memory(uint32_t count, uint32_t size, unsigned mode)
+void intel_require_memory(uint64_t count, uint64_t size, unsigned mode)
 {
-/* rough estimate of how many bytes the kernel requires to track each object */
-#define KERNEL_BO_OVERHEAD 512
 	uint64_t required, total;
 
-	required = count;
-	required *= size + KERNEL_BO_OVERHEAD;
-	required = ALIGN(required, 4096);
-
-	igt_debug("Checking %u surfaces of size %u bytes (total %llu) against %s%s\n",
-		  count, size, (long long)required,
-		  mode & (CHECK_RAM | CHECK_SWAP) ? "RAM" : "",
-		  mode & CHECK_SWAP ? " + swap": "");
-
-	total = 0;
-	if (mode & (CHECK_RAM | CHECK_SWAP))
-		total += intel_get_avail_ram_mb();
-	if (mode & CHECK_SWAP)
-		total += intel_get_total_swap_mb();
-	total *= 1024 * 1024;
-
-	igt_skip_on_f(total <= required,
-		      "Estimated that we need %llu bytes for the test, but only have %llu bytes available (%s%s)\n",
-		      (long long)required, (long long)total,
+	igt_require_f(__intel_check_memory(count, size, mode,
+					    &required, &total),
+		      "Estimated that we need %'llu objects and %'llu MiB for the test, but only have %'llu MiB available (%s%s) and a maximum of %'llu objects\n",
+		      (long long)count,
+		      (long long)((required + ((1<<20) - 1)) >> 20),
+		      (long long)(total >> 20),
 		      mode & (CHECK_RAM | CHECK_SWAP) ? "RAM" : "",
-		      mode & CHECK_SWAP ? " + swap": "");
+		      mode & CHECK_SWAP ? " + swap": "",
+		      (long long)vfs_file_max());
 
 	igt_skip_on_simulation();
 }
 
-void
-intel_purge_vm_caches(void)
+void intel_purge_vm_caches(void)
 {
 	int fd;
 
-	fd = open("/proc/sys/vm/drop_caches", O_RDWR);
+	fd = open("/proc/sys/vm/drop_caches", O_WRONLY);
 	if (fd < 0)
 		return;
 
-	igt_assert_eq(write(fd, "3\n", 2), 2);
+	/* BIT(0): Drop page cache
+	 * BIT(1): Drop slab cache
+	 * BIT(2): Be quiet in future
+	 */
+	igt_ignore_warn(write(fd, "7\n", 2));
 	close(fd);
 }
 
