@@ -26,21 +26,18 @@
  *
  */
 
+#include "igt.h"
 #include <limits.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#include "intel_chipset.h"
-#include "drmtest.h"
-#include "igt_aux.h"
-#include "igt_gt.h"
-#include "ioctl_wrappers.h"
-
 #ifndef I915_PARAM_CMD_PARSER_VERSION
 #define I915_PARAM_CMD_PARSER_VERSION       28
 #endif
 
+static char read_buffer[1024];
+
 static int _read_sysfs(void *dst, int maxlen,
 		      const char* path,
 		      const char *fname)
@@ -77,9 +74,7 @@ static int read_sysfs(void *dst, int maxlen, const char *fname)
 
 static void test_sysfs_error_exists(void)
 {
-	char tmp[1024];
-
-	igt_assert_lt(0, read_sysfs(tmp, sizeof(tmp), "error"));
+	igt_assert_lt(0, read_sysfs(read_buffer, sizeof(read_buffer), "error"));
 }
 
 static void test_debugfs_error_state_exists(void)
@@ -92,15 +87,6 @@ static void test_debugfs_error_state_exists(void)
 	close (fd);
 }
 
-static void test_debugfs_ring_stop_exists(void)
-{
-	int fd;
-
-	igt_assert_lte(0, (fd = igt_debugfs_open("i915_ring_stop", O_RDONLY)));
-
-	close(fd);
-}
-
 static void read_dfs(const char *fname, char *d, int maxlen)
 {
 	int fd;
@@ -116,29 +102,26 @@ static void read_dfs(const char *fname, char *d, int maxlen)
 	igt_debug("dfs entry %s read '%s'\n", fname, d);
 }
 
-static void _assert_dfs_entry(const char *fname, const char *s, bool inverse)
+static int compare_dfs_entry(const char *fname, const char *s)
 {
-	char tmp[1024];
-	const int l = min(strlen(s), sizeof(tmp));
+	const int l = min(strlen(s), sizeof(read_buffer));
 
-	read_dfs(fname, tmp, l + 1);
-	if (!inverse) {
-		igt_fail_on_f(strncmp(tmp, s, l) != 0,
-			      "contents of %s: '%s' (expected '%s')\n", fname, tmp, s);
-	} else {
-		igt_fail_on_f(strncmp(tmp, s, l) == 0,
-			      "contents of %s: '%s' (expected not '%s'\n", fname, tmp, s);
-	}
+	read_dfs(fname, read_buffer, l + 1);
+	return strncmp(read_buffer, s, l);
 }
 
 static void assert_dfs_entry(const char *fname, const char *s)
 {
-	_assert_dfs_entry(fname, s, false);
+	igt_fail_on_f(compare_dfs_entry(fname, s) != 0,
+		      "contents of %s: '%s' (expected '%s')\n",
+		      fname, read_buffer, s);
 }
 
 static void assert_dfs_entry_not(const char *fname, const char *s)
 {
-	_assert_dfs_entry(fname, s, true);
+	igt_fail_on_f(compare_dfs_entry(fname, s) == 0,
+		      "contents of %s: '%s' (expected not '%s'\n",
+		      fname, read_buffer, s);
 }
 
 static void assert_error_state_clear(void)
@@ -151,59 +134,21 @@ static void assert_error_state_collected(void)
 	assert_dfs_entry_not("i915_error_state", "no error state collected");
 }
 
-#define MAGIC_NUMBER 0x10001
-const uint32_t batch[] = { MI_NOOP,
-			   MI_BATCH_BUFFER_END,
-			   MAGIC_NUMBER,
-			   MAGIC_NUMBER };
+const uint32_t *batch;
 
-static uint64_t submit_batch(int fd, unsigned ring_id, bool stop_ring)
+static uint64_t submit_hang(int fd, unsigned ring_id)
 {
-	struct drm_i915_gem_execbuffer2 execbuf;
-	struct drm_i915_gem_exec_object2 exec;
-	uint64_t presumed_offset;
+	uint64_t offset;
+	igt_hang_ring_t hang;
 
-	gem_require_ring(fd, ring_id);
+	hang = igt_hang_ctx(fd, 0, ring_id, HANG_ALLOW_CAPTURE, &offset);
 
-	exec.handle = gem_create(fd, 4096);
-	gem_write(fd, exec.handle, 0, batch, sizeof(batch));
-	exec.relocation_count = 0;
-	exec.relocs_ptr = 0;
-	exec.alignment = 0;
-	exec.offset = 0;
-	exec.flags = 0;
-	exec.rsvd1 = 0;
-	exec.rsvd2 = 0;
+	batch = gem_mmap__cpu(fd, hang.handle, 0, 4096, PROT_READ);
+	gem_set_domain(fd, hang.handle, I915_GEM_DOMAIN_CPU, 0);
 
-	execbuf.buffers_ptr = (uintptr_t)&exec;
-	execbuf.buffer_count = 1;
-	execbuf.batch_start_offset = 0;
-	execbuf.batch_len = sizeof(batch);
-	execbuf.cliprects_ptr = 0;
-	execbuf.num_cliprects = 0;
-	execbuf.DR1 = 0;
-	execbuf.DR4 = 0;
-	execbuf.flags = ring_id;
-	i915_execbuffer2_set_context_id(execbuf, 0);
-	execbuf.rsvd2 = 0;
+	igt_post_hang_ring(fd, hang);
 
-	gem_execbuf(fd, &execbuf);
-	gem_sync(fd, exec.handle);
-	presumed_offset = exec.offset;
-
-	if (stop_ring) {
-		igt_set_stop_rings(igt_to_stop_ring_flag(ring_id));
-
-		gem_execbuf(fd, &execbuf);
-		gem_sync(fd, exec.handle);
-
-		igt_assert(igt_get_stop_rings() == STOP_RING_NONE);
-		igt_assert(presumed_offset == exec.offset);
-	}
-
-	gem_close(fd, exec.handle);
-
-	return exec.offset;
+	return offset;
 }
 
 static void clear_error_state(void)
@@ -221,11 +166,12 @@ static void test_error_state_basic(void)
 {
 	int fd;
 
+	fd = drm_open_driver(DRIVER_INTEL);
+
 	clear_error_state();
 	assert_error_state_clear();
 
-	fd = drm_open_any();
-	submit_batch(fd, I915_EXEC_RENDER, true);
+	submit_hang(fd, I915_EXEC_RENDER);
 	close(fd);
 
 	assert_error_state_collected();
@@ -239,111 +185,53 @@ static void check_error_state(const int gen,
 			      uint64_t expected_offset)
 {
 	FILE *file;
-	int debug_fd;
 	char *line = NULL;
 	size_t line_size = 0;
-	char *ring_name = NULL;
-	bool bb_ok = false, req_ok = false, ringbuf_ok = false;
 
-	debug_fd = igt_debugfs_open("i915_error_state", O_RDONLY);
-	igt_assert_lte(0, debug_fd);
-	file = fdopen(debug_fd, "r");
+	file = igt_debugfs_fopen("i915_error_state", "r");
+	igt_require(file);
 
 	while (getline(&line, &line_size, file) > 0) {
-		char *dashes = NULL;
-		int bb_matched = 0;
-		uint32_t gtt_offset;
-		int req_matched = 0;
-		int requests;
-		uint32_t tail;
-		int ringbuf_matched = 0;
-		int i, items;
+		char *dashes;
+		uint32_t gtt_offset_upper, gtt_offset_lower;
+		int matched;
 
 		dashes = strstr(line, "---");
 		if (!dashes)
 			continue;
 
-		ring_name = realloc(ring_name, dashes - line);
-		strncpy(ring_name, line, dashes - line);
-		ring_name[dashes - line - 1] = '\0';
+		matched = sscanf(dashes, "--- gtt_offset = 0x%08x %08x\n",
+				    &gtt_offset_upper, &gtt_offset_lower);
+		if (matched) {
+			char expected_line[64];
+			uint64_t gtt_offset;
+			int i;
 
-		bb_matched = sscanf(dashes, "--- gtt_offset = 0x%08x\n",
-				    &gtt_offset);
-		if (bb_matched == 1) {
-			char expected_line[32];
+			strncpy(expected_line, line, dashes - line);
+			expected_line[dashes - line - 1] = '\0';
+			igt_assert(strstr(expected_line, expected_ring_name));
 
-			igt_assert(strstr(ring_name, expected_ring_name));
+			gtt_offset = gtt_offset_upper;
+			if (matched == 2) {
+				gtt_offset <<= 32;
+				gtt_offset |= gtt_offset_lower;
+			}
 			if (!uses_cmd_parser)
-				igt_assert(gtt_offset == expected_offset);
+				igt_assert_eq_u64(gtt_offset, expected_offset);
 
-			for (i = 0; i < sizeof(batch) / 4; i++) {
+			for (i = 0; i < 1024; i++) {
 				igt_assert(getline(&line, &line_size, file) > 0);
-				snprintf(expected_line, sizeof(expected_line), "%08x :  %08x",
+				snprintf(expected_line, sizeof(expected_line),
+					 "%08x :  %08x",
 					 4*i, batch[i]);
 				igt_assert(strstr(line, expected_line));
 			}
-			bb_ok = true;
-			continue;
-		}
-
-		req_matched = sscanf(dashes, "--- %d requests\n", &requests);
-		if (req_matched == 1) {
-			igt_assert(strstr(ring_name, expected_ring_name));
-			igt_assert_lt(0, requests);
-
-			for (i = 0; i < requests; i++) {
-				uint32_t seqno;
-				long jiffies;
-
-				igt_assert(getline(&line, &line_size, file) > 0);
-				items = sscanf(line, "  seqno 0x%08x, emitted %ld, tail 0x%08x\n",
-					       &seqno, &jiffies, &tail);
-				igt_assert_eq(items, 3);
-			}
-			req_ok = true;
-			continue;
-		}
-
-		ringbuf_matched = sscanf(dashes, "--- ringbuffer = 0x%08x\n",
-					 &gtt_offset);
-		if (ringbuf_matched == 1) {
-			unsigned int offset, command, expected_addr = 0;
-
-			if (!strstr(ring_name, expected_ring_name))
-				continue;
-			igt_assert(req_ok);
-
-			for (i = 0; i < tail / 4; i++) {
-				igt_assert(getline(&line, &line_size, file) > 0);
-				items = sscanf(line, "%08x :  %08x\n",
-					       &offset, &command);
-				igt_assert_eq(items, 2);
-				if ((command & 0x1F800000) == MI_BATCH_BUFFER_START) {
-					igt_assert(getline(&line, &line_size, file) > 0);
-					items = sscanf(line, "%08x :  %08x\n",
-						       &offset, &expected_addr);
-					igt_assert_eq(items, 2);
-					i++;
-				}
-			}
-			if (!uses_cmd_parser) {
-				if (gen >= 4)
-					igt_assert(expected_addr == expected_offset);
-				else
-					igt_assert((expected_addr & ~0x1) == expected_offset);
-			}
-			ringbuf_ok = true;
-			continue;
-		}
-
-		if (bb_ok && req_ok && ringbuf_ok)
 			break;
+		}
 	}
-	igt_assert(bb_ok && req_ok && ringbuf_ok);
 
 	free(line);
-	free(ring_name);
-	close(debug_fd);
+	fclose(file);
 }
 
 static bool uses_cmd_parser(int fd, int gen)
@@ -358,7 +246,7 @@ static bool uses_cmd_parser(int fd, int gen)
 	if (rc || parser_version == 0)
 		return false;
 
-	if (!gem_uses_aliasing_ppgtt(fd))
+	if (!gem_uses_ppgtt(fd))
 		return false;
 
 	if (gen != 7)
@@ -374,31 +262,63 @@ static void test_error_state_capture(unsigned ring_id,
 	uint64_t offset;
 	bool cmd_parser;
 
+	fd = drm_open_driver(DRIVER_INTEL);
+
 	clear_error_state();
 
-	fd = drm_open_any();
 	gen = intel_gen(intel_get_drm_devid(fd));
 	cmd_parser = uses_cmd_parser(fd, gen);
 
-	offset = submit_batch(fd, ring_id, true);
+	offset = submit_hang(fd, ring_id);
 	close(fd);
 
 	check_error_state(gen, cmd_parser, ring_name, offset);
 }
 
-static const struct target_ring {
-	const int id;
-	const char *short_name;
-	const char *full_name;
-} rings[] = {
-	{ I915_EXEC_RENDER, "render", "render ring" },
-	{ I915_EXEC_BSD, "bsd", "bsd ring" },
-	{ I915_EXEC_BLT, "blt", "blitter ring" },
-	{ I915_EXEC_VEBOX, "vebox", "video enhancement ring" },
-};
+
+/* This test covers the case where we end up in an uninitialised area of the
+ * ppgtt and keep executing through it. This is particularly relevant if 48b
+ * ppgtt is enabled because the ppgtt is massively bigger compared to the 32b
+ * case and it takes a lot more time to wrap, so the acthd can potentially keep
+ * increasing for a long time
+ */
+#define NSEC_PER_SEC	1000000000LL
+static void hangcheck_unterminated(void)
+{
+	int fd;
+	/* timeout needs to be greater than ~5*hangcheck */
+	int64_t timeout_ns = 100 * NSEC_PER_SEC; /* 100 seconds */
+	struct drm_i915_gem_execbuffer2 execbuf;
+	struct drm_i915_gem_exec_object2 gem_exec;
+	uint32_t handle;
+
+	fd = drm_open_driver(DRIVER_INTEL);
+	igt_require(gem_uses_full_ppgtt(fd));
+	igt_require_hang_ring(fd, 0);
+
+	handle = gem_create(fd, 4096);
+
+	memset(&gem_exec, 0, sizeof(gem_exec));
+	gem_exec.handle = handle;
+
+	memset(&execbuf, 0, sizeof(execbuf));
+	execbuf.buffers_ptr = (uintptr_t)&gem_exec;
+	execbuf.buffer_count = 1;
+
+	gem_execbuf(fd, &execbuf);
+	if (gem_wait(fd, handle, &timeout_ns) != 0) {
+		/* need to manually trigger an hang to clean before failing */
+		igt_force_gpu_reset();
+		igt_assert_f(0, "unterminated batch did not trigger an hang!");
+	}
+
+	close(fd);
+}
 
 igt_main
 {
+	const struct intel_execution_engine *e;
+
 	igt_skip_on_simulation();
 
 	igt_subtest("error-state-debugfs-entry")
@@ -407,14 +327,18 @@ igt_main
 	igt_subtest("error-state-sysfs-entry")
 		test_sysfs_error_exists();
 
-	igt_subtest("ring-stop-sysfs-entry")
-		test_debugfs_ring_stop_exists();
-
 	igt_subtest("error-state-basic")
 		test_error_state_basic();
 
-	for (int i = 0; i < sizeof(rings)/sizeof(rings[0]); i++) {
-		igt_subtest_f("error-state-capture-%s", rings[i].short_name)
-			test_error_state_capture(rings[i].id, rings[i].full_name);
+	for (e = intel_execution_engines; e->name; e++) {
+		if (e->exec_id == 0)
+			continue;
+
+		igt_subtest_f("error-state-capture-%s", e->name)
+			test_error_state_capture(e->exec_id | e->flags,
+						 e->full_name);
 	}
+
+	igt_subtest("hangcheck-unterminated")
+		hangcheck_unterminated();
 }
