@@ -132,11 +132,11 @@ struct event_state {
 static void dump_event_state(const struct event_state *es)
 {
 	igt_debug("name = %s\n"
-		  "last_ts = %ld.%ld usec\n"
-		  "last_received_ts = %ld.%ld usec\n"
+		  "last_ts = %ld.%06ld\n"
+		  "last_received_ts = %ld.%06ld\n"
 		  "last_seq = %u\n"
-		  "current_ts = %ld.%ld usec\n"
-		  "current_received_ts = %ld.%ld usec\n"
+		  "current_ts = %ld.%06ld\n"
+		  "current_received_ts = %ld.%06ld\n"
 		  "current_seq = %u\n"
 		  "count = %u\n"
 		  "seq_step = %d\n",
@@ -481,15 +481,13 @@ static int __wait_for_vblank(unsigned int flags, int crtc_idx,
 {
 	drmVBlank wait_vbl;
 	int ret;
-	unsigned crtc_idx_mask;
+	uint32_t pipe_id_flag;
 	bool event = !(flags & TEST_VBLANK_BLOCK);
 
 	memset(&wait_vbl, 0, sizeof(wait_vbl));
+	pipe_id_flag = kmstest_get_vbl_flag(crtc_idx);
 
-	crtc_idx_mask = crtc_idx << DRM_VBLANK_HIGH_CRTC_SHIFT;
-	igt_assert(!(crtc_idx_mask & ~DRM_VBLANK_HIGH_CRTC_MASK));
-
-	wait_vbl.request.type = crtc_idx_mask;
+	wait_vbl.request.type = pipe_id_flag;
 	if (flags & TEST_VBLANK_ABSOLUTE)
 		wait_vbl.request.type |= DRM_VBLANK_ABSOLUTE;
 	else
@@ -531,7 +529,7 @@ static int do_wait_for_vblank(struct test_output *o, int pipe_id,
 }
 
 static bool
-analog_tv_connector(struct test_output *o)
+analog_tv_connector(const struct test_output *o)
 {
 	uint32_t connector_type = o->kconnector[0]->connector_type;
 
@@ -574,7 +572,7 @@ static void page_flip_handler(int fd, unsigned int frame, unsigned int sec,
 		event_handler(&o->flip_state, frame, sec, usec);
 }
 
-static double frame_time(struct test_output *o)
+static double frame_time(const struct test_output *o)
 {
 	return 1000.0 * o->kmode[0].htotal * o->kmode[0].vtotal / o->kmode[0].clock;
 }
@@ -640,17 +638,16 @@ static void vblank_handler(int fd, unsigned int frame, unsigned int sec,
 	fixup_premature_vblank_ts(o, &o->vblank_state);
 }
 
-static void check_state(struct test_output *o, struct event_state *es)
+static void check_state(const struct test_output *o, const struct event_state *es)
 {
 	struct timeval diff;
-	double usec_interflip;
 
 	dump_event_state(es);
 
 	timersub(&es->current_ts, &es->current_received_ts, &diff);
 	if (!analog_tv_connector(o)) {
 		igt_assert_f(diff.tv_sec < 0 || (diff.tv_sec == 0 && diff.tv_usec <= 2000),
-			     "%s ts delayed for too long: %lds, %ldusec\n",
+			     "%s ts delayed for too long: %ld.%06ld\n",
 			     es->name, diff.tv_sec, diff.tv_usec);
 
 	}
@@ -661,7 +658,7 @@ static void check_state(struct test_output *o, struct event_state *es)
 	timersub(&es->current_ts, &es->last_received_ts, &diff);
 	igt_assert_f(timercmp(&es->last_received_ts, &es->current_ts, <),
 		     "%s ts before the %s was issued!\n"
-		     "timerdiff %lds, %ldusec\n",
+		     "timerdiff %ld.%06ld\n",
 		     es->name, es->name,
 		     diff.tv_sec, diff.tv_usec);
 
@@ -689,13 +686,24 @@ static void check_state(struct test_output *o, struct event_state *es)
 	}
 
 	if ((o->flags & TEST_CHECK_TS) && (!analog_tv_connector(o))) {
-		timersub(&es->current_ts, &es->last_ts, &diff);
-		usec_interflip = (double)o->seq_step * frame_time(o);
+		double usec_interflip, usec_diff;
 
-		igt_assert_f(fabs((((double) diff.tv_usec) - usec_interflip) /
+		timersub(&es->current_ts, &es->last_ts, &diff);
+		usec_diff = diff.tv_sec * USEC_PER_SEC + diff.tv_usec;
+
+		usec_interflip = (es->current_seq - es->last_seq) * frame_time(o);
+		igt_assert_f(fabs((usec_diff - usec_interflip) /
 				  usec_interflip) <= 0.005,
-			     "inter-%s ts jitter: %lds, %ldusec\n",
-			     es->name, diff.tv_sec, diff.tv_usec);
+			     "inconsistent %s ts/seq: last %ld.%06ld/%u, current %ld.%06ld/%u\n",
+			     es->name, es->last_ts.tv_sec, es->last_ts.tv_usec, es->last_seq,
+			     es->current_ts.tv_sec, es->current_ts.tv_usec, es->current_seq);
+
+		usec_interflip = o->seq_step * frame_time(o);
+		igt_assert_f(fabs((usec_diff - usec_interflip) /
+				  usec_interflip) <= 0.005,
+			     "inter-%s ts jitter: %ld.%06ld, expected %.6f\n",
+			     es->name, diff.tv_sec, diff.tv_usec,
+			     usec_interflip / USEC_PER_SEC);
 
 		igt_assert_f(es->current_seq == es->last_seq + o->seq_step,
 			     "unexpected %s seq %u, expected %u\n",
@@ -717,15 +725,15 @@ static void check_state_correlation(struct test_output *o,
 		return;
 
 	timersub(&es2->current_ts, &es1->current_ts, &tv_diff);
-	usec_diff = tv_diff.tv_sec * 1000 * 1000 + tv_diff.tv_usec;
+	usec_diff = tv_diff.tv_sec * USEC_PER_SEC + tv_diff.tv_usec;
 
 	seq_diff = es2->current_seq - es1->current_seq;
 	ftime = frame_time(o);
 	usec_diff -= seq_diff * ftime;
 
 	igt_assert_f(fabs(usec_diff) / ftime <= 0.005,
-		     "timestamp mismatch between %s and %s (diff %.4f sec)\n",
-		     es1->name, es2->name, usec_diff / 1000 / 1000);
+		     "timestamp mismatch between %s and %s (diff %.6f sec)\n",
+		     es1->name, es2->name, usec_diff / USEC_PER_SEC);
 }
 
 static void check_all_state(struct test_output *o,
@@ -1173,7 +1181,8 @@ fb_is_bound(struct test_output *o, int fb)
 	return true;
 }
 
-static void check_final_state(struct test_output *o, struct event_state *es,
+static void check_final_state(const struct test_output *o,
+			      const struct event_state *es,
 			      unsigned int elapsed)
 {
 	igt_assert_f(es->count > 0,
@@ -1343,8 +1352,10 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 
 	/* 256 MB is usually the maximum mappable aperture,
 	 * (make it 4x times that to ensure failure) */
-	if (o->flags & TEST_BO_TOOBIG)
-		bo_size = 4*256*1024*1024;
+	if (o->flags & TEST_BO_TOOBIG) {
+		bo_size = 4*gem_mappable_aperture_size();
+		igt_require(bo_size < gem_global_aperture_size(drm_fd));
+	}
 
 	o->fb_ids[0] = igt_create_fb(drm_fd, o->fb_width, o->fb_height,
 					 igt_bpp_depth_to_drm_format(o->bpp, o->depth),
@@ -1392,11 +1403,7 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	if (o->flags & TEST_CHECK_TS)
 		sleep(1);
 
-	if (o->flags & TEST_BO_TOOBIG) {
-		igt_assert_eq(do_page_flip(o, o->fb_ids[1], true), -E2BIG);
-		goto out;
-	} else
-		igt_assert_eq(do_page_flip(o, o->fb_ids[1], true), 0);
+	igt_assert_eq(do_page_flip(o, o->fb_ids[1], true), 0);
 	wait_for_events(o);
 
 	o->current_fb_id = 1;
