@@ -97,12 +97,15 @@ struct test_mode {
 		FBS_COUNT,
 	} fbs;
 
-	/* Which features are we going to test now? This is a mask! */
+	/* Which features are we going to test now? This is a mask!
+	 * FEATURE_DEFAULT is a special value which instruct the test to just
+	 * keep what's already enabled by default in the Kernel. */
 	enum {
 		FEATURE_NONE  = 0,
 		FEATURE_FBC   = 1,
 		FEATURE_PSR   = 2,
 		FEATURE_COUNT = 4,
+		FEATURE_DEFAULT = 4,
 	} feature;
 
 	/* Possible pixel formats. We just use FORMAT_DEFAULT for most tests and
@@ -149,10 +152,12 @@ struct rect {
 
 #define MAX_CONNECTORS 32
 #define MAX_PLANES 32
+#define MAX_ENCODERS 32
 struct {
 	int fd;
 	drmModeResPtr res;
 	drmModeConnectorPtr connectors[MAX_CONNECTORS];
+	drmModeEncoderPtr encoders[MAX_ENCODERS];
 	drmModePlaneResPtr plane_res;
 	drmModePlanePtr planes[MAX_PLANES];
 	uint64_t plane_types[MAX_PLANES];
@@ -354,32 +359,45 @@ static drmModeConnectorPtr get_connector(uint32_t id)
 	igt_assert(false);
 }
 
+static drmModeEncoderPtr get_encoder(uint32_t id)
+{
+	int i;
+
+	for (i = 0; i < drm.res->count_encoders; i++)
+		if (drm.res->encoders[i] == id)
+			return drm.encoders[i];
+
+	igt_assert(false);
+}
+
 static void print_mode_info(const char *screen, struct modeset_params *params)
 {
 	drmModeConnectorPtr c = get_connector(params->connector_id);
 
-	igt_info("%s screen: %s %s\n",
+	igt_info("%s screen: %s %s, crtc %d\n",
 		 screen,
 		 kmstest_connector_type_str(c->connector_type),
-		 params->mode->name);
+		 params->mode->name,
+		 params->crtc_id);
 }
 
 static void init_mode_params(struct modeset_params *params, uint32_t crtc_id,
-			     int crtc_index, uint32_t connector_id,
+			     drmModeConnectorPtr connector,
 			     drmModeModeInfoPtr mode)
 {
 	uint32_t plane_id = 0;
+	int crtc_idx = kmstest_get_crtc_idx(drm.res, crtc_id);
 	int i;
 
 	for (i = 0; i < drm.plane_res->count_planes && plane_id == 0; i++)
-		if ((drm.planes[i]->possible_crtcs & (1 << crtc_index)) &&
+		if ((drm.planes[i]->possible_crtcs & (1 << crtc_idx)) &&
 		    drm.plane_types[i] == DRM_PLANE_TYPE_OVERLAY)
 			plane_id = drm.planes[i]->plane_id;
 
 	igt_assert(plane_id);
 
 	params->crtc_id = crtc_id;
-	params->connector_id = connector_id;
+	params->connector_id = connector->connector_id;
 	params->mode = mode;
 	params->sprite_id = plane_id;
 
@@ -424,56 +442,90 @@ static bool connector_get_mode(drmModeConnectorPtr c, drmModeModeInfoPtr *mode)
 	return true;
 }
 
-static bool init_modeset_cached_params(void)
+static bool connector_supports_pipe_a(drmModeConnectorPtr connector)
 {
 	int i;
-	uint32_t prim_connector_id = 0, scnd_connector_id = 0;
+
+	for (i = 0; i < connector->count_encoders; i++)
+		if (get_encoder(connector->encoders[i])->possible_crtcs & 1)
+			return true;
+
+	return false;
+}
+
+static bool find_connector(bool edp_only, bool pipe_a, uint32_t forbidden_id,
+			   drmModeConnectorPtr *ret_connector,
+			   drmModeModeInfoPtr *ret_mode)
+{
+	drmModeConnectorPtr c = NULL;
+	drmModeModeInfoPtr mode = NULL;
+	int i;
+
+	for (i = 0; i < drm.res->count_connectors; i++) {
+		c = drm.connectors[i];
+
+		if (edp_only && c->connector_type != DRM_MODE_CONNECTOR_eDP)
+			continue;
+		if (pipe_a && !connector_supports_pipe_a(c))
+			continue;
+		if (c->connector_id == forbidden_id)
+			continue;
+		if (!connector_get_mode(c, &mode))
+			continue;
+
+		*ret_connector = c;
+		*ret_mode = mode;
+		return true;
+	}
+
+	return false;
+}
+
+static bool init_modeset_cached_params(void)
+{
+	drmModeConnectorPtr prim_connector = NULL, scnd_connector = NULL;
 	drmModeModeInfoPtr prim_mode = NULL, scnd_mode = NULL;
-	drmModeModeInfoPtr tmp_mode;
+	uint32_t prim_crtc_id, scnd_crtc_id;
 
-	/* First, try to find an eDP monitor since it's the only possible type
-	 * for PSR.  */
-	for (i = 0; i < drm.res->count_connectors; i++) {
-		if (drm.connectors[i]->connector_type != DRM_MODE_CONNECTOR_eDP)
-			continue;
+	/*
+	 * We have this problem where PSR is only present on eDP monitors and
+	 * FBC is only present on pipe A for some platforms. So we search first
+	 * for the ideal case of eDP supporting pipe A, and try the less optimal
+	 * configs later, sacrificing  one of the features.
+	 * TODO: refactor the code in a way that allows us to have different
+	 * sets of prim/scnd structs for different features.
+	 */
+	find_connector(true, true, 0, &prim_connector, &prim_mode);
+	if (!prim_connector)
+		find_connector(true, false, 0, &prim_connector, &prim_mode);
+	if (!prim_connector)
+		find_connector(false, true, 0, &prim_connector, &prim_mode);
+	if (!prim_connector)
+		find_connector(false, false, 0, &prim_connector, &prim_mode);
 
-		if (connector_get_mode(drm.connectors[i], &tmp_mode)) {
-			prim_connector_id = drm.res->connectors[i];
-			prim_mode = tmp_mode;
-		}
-	}
-	for (i = 0; i < drm.res->count_connectors; i++) {
-		/* Don't pick again what we just selected on the above loop. */
-		if (drm.res->connectors[i] == prim_connector_id)
-			continue;
-
-		if (connector_get_mode(drm.connectors[i], &tmp_mode)) {
-			if (!prim_connector_id) {
-				prim_connector_id = drm.res->connectors[i];
-				prim_mode = tmp_mode;
-			} else if (!scnd_connector_id) {
-				scnd_connector_id = drm.res->connectors[i];
-				scnd_mode = tmp_mode;
-				break;
-			}
-		}
-	}
-
-	if (!prim_connector_id)
+	if (!prim_connector)
 		return false;
 
-	init_mode_params(&prim_mode_params, drm.res->crtcs[0], 0,
-			 prim_connector_id, prim_mode);
+	find_connector(false, false, prim_connector->connector_id,
+		       &scnd_connector, &scnd_mode);
+
+	prim_crtc_id = kmstest_find_crtc_for_connector(drm.fd, drm.res,
+						       prim_connector, 0);
+	init_mode_params(&prim_mode_params, prim_crtc_id,
+			 prim_connector, prim_mode);
 	print_mode_info("Primary", &prim_mode_params);
 
-	if (!scnd_connector_id) {
+	if (!scnd_connector) {
 		scnd_mode_params.connector_id = 0;
 		return true;
 	}
 
 	igt_assert(drm.res->count_crtcs >= 2);
-	init_mode_params(&scnd_mode_params, drm.res->crtcs[1], 1,
-			 scnd_connector_id, scnd_mode);
+	scnd_crtc_id = kmstest_find_crtc_for_connector(drm.fd, drm.res,
+						      scnd_connector,
+			1 << kmstest_get_crtc_idx(drm.res, prim_crtc_id));
+	init_mode_params(&scnd_mode_params, scnd_crtc_id,
+			 scnd_connector, scnd_mode);
 	print_mode_info("Secondary", &scnd_mode_params);
 
 	return true;
@@ -483,6 +535,9 @@ static void create_fb(enum pixel_format pformat, int width, int height,
 		      uint64_t tiling, int plane, struct igt_fb *fb)
 {
 	uint32_t format;
+	unsigned int size, stride;
+	int bpp;
+	uint64_t tiling_for_size;
 
 	switch (pformat) {
 	case FORMAT_RGB888:
@@ -512,7 +567,21 @@ static void create_fb(enum pixel_format pformat, int width, int height,
 		igt_assert(false);
 	}
 
-	igt_create_fb(drm.fd, width, height, format, tiling, fb);
+	/* We want all frontbuffers with the same width/height/format to have
+	 * the same size regardless of tiling since we want to properly exercise
+	 * the Kernel's specific tiling-checking code paths without accidentally
+	 * hitting size-checking ones first. */
+	bpp = igt_drm_format_to_bpp(format);
+	if (plane == PLANE_CUR)
+		tiling_for_size = LOCAL_DRM_FORMAT_MOD_NONE;
+	else
+		tiling_for_size = LOCAL_I915_FORMAT_MOD_X_TILED;
+
+	igt_calc_fb_size(drm.fd, width, height, bpp, tiling_for_size, &size,
+			 &stride);
+
+	igt_create_fb_with_bo_size(drm.fd, width, height, format, tiling, fb,
+				   size, stride);
 }
 
 static uint32_t pick_color(struct igt_fb *fb, enum color ecolor)
@@ -642,12 +711,31 @@ static void create_shared_fb(enum pixel_format format)
 		  PLANE_PRI, &s->big);
 }
 
+static void destroy_fbs(enum pixel_format format)
+{
+	struct screen_fbs *s = &fbs[format];
+
+	if (!s->initialized)
+		return;
+
+	if (scnd_mode_params.connector_id) {
+		igt_remove_fb(drm.fd, &s->scnd_pri);
+		igt_remove_fb(drm.fd, &s->scnd_cur);
+		igt_remove_fb(drm.fd, &s->scnd_spr);
+	}
+	igt_remove_fb(drm.fd, &s->prim_pri);
+	igt_remove_fb(drm.fd, &s->prim_cur);
+	igt_remove_fb(drm.fd, &s->prim_spr);
+	igt_remove_fb(drm.fd, &s->offscreen);
+	igt_remove_fb(drm.fd, &s->big);
+}
+
 static void create_fbs(enum pixel_format format)
 {
 	struct screen_fbs *s = &fbs[format];
 
 	if (s->initialized)
-		return;
+		destroy_fbs(format);
 
 	s->initialized = true;
 
@@ -678,25 +766,6 @@ static void create_fbs(enum pixel_format format)
 		  LOCAL_I915_FORMAT_MOD_X_TILED, PLANE_SPR, &s->scnd_spr);
 }
 
-static void destroy_fbs(enum pixel_format format)
-{
-	struct screen_fbs *s = &fbs[format];
-
-	if (!s->initialized)
-		return;
-
-	if (scnd_mode_params.connector_id) {
-		igt_remove_fb(drm.fd, &s->scnd_pri);
-		igt_remove_fb(drm.fd, &s->scnd_cur);
-		igt_remove_fb(drm.fd, &s->scnd_spr);
-	}
-	igt_remove_fb(drm.fd, &s->prim_pri);
-	igt_remove_fb(drm.fd, &s->prim_cur);
-	igt_remove_fb(drm.fd, &s->prim_spr);
-	igt_remove_fb(drm.fd, &s->offscreen);
-	igt_remove_fb(drm.fd, &s->big);
-}
-
 static bool set_mode_for_params(struct modeset_params *params)
 {
 	int rc;
@@ -851,7 +920,7 @@ static bool fbc_wait_until_enabled(void)
 
 static bool psr_wait_until_enabled(void)
 {
-	return igt_wait(psr_is_enabled(), 2000, 1);
+	return igt_wait(psr_is_enabled(), 5000, 1);
 }
 
 #define fbc_enable() igt_set_module_param_int("enable_fbc", 1)
@@ -859,12 +928,22 @@ static bool psr_wait_until_enabled(void)
 #define psr_enable() igt_set_module_param_int("enable_psr", 1)
 #define psr_disable() igt_set_module_param_int("enable_psr", 0)
 
-static void get_sink_crc(sink_crc_t *crc)
+static void get_sink_crc(sink_crc_t *crc, bool mandatory)
 {
+	int rc, errno_;
+
 	lseek(sink_crc.fd, 0, SEEK_SET);
 
-	igt_assert(read(sink_crc.fd, crc->data, SINK_CRC_SIZE) ==
-		   SINK_CRC_SIZE);
+	rc = read(sink_crc.fd, crc->data, SINK_CRC_SIZE);
+	errno_ = errno;
+
+	if (rc == -1 && errno_ == ETIMEDOUT) {
+		if (mandatory)
+			igt_skip("Sink CRC is unreliable on this machine. Try running this test again individually\n");
+		else
+			igt_info("Sink CRC is unreliable on this machine. Try running this test again individually\n");
+	}
+	igt_assert(rc == SINK_CRC_SIZE);
 }
 
 static bool sink_crc_equal(sink_crc_t *a, sink_crc_t *b)
@@ -1049,7 +1128,7 @@ static void fill_fb_region(struct fb_region *region, enum color ecolor)
 {
 	uint32_t color = pick_color(region->fb, ecolor);
 
-	igt_draw_rect_fb(drm.fd, NULL, NULL, region->fb, IGT_DRAW_MMAP_CPU,
+	igt_draw_rect_fb(drm.fd, drm.bufmgr, NULL, region->fb, IGT_DRAW_BLT,
 			 region->x, region->y, region->w, region->h,
 			 color);
 }
@@ -1074,8 +1153,11 @@ static void unset_all_crtcs(void)
 	}
 }
 
-static void disable_features(void)
+static void disable_features(const struct test_mode *t)
 {
+	if (t->feature == FEATURE_DEFAULT)
+		return;
+
 	fbc_disable();
 	psr_disable();
 }
@@ -1092,21 +1174,6 @@ static void *busy_thread_func(void *data)
 	pthread_exit(0);
 }
 
-static int fb_get_bpp(struct igt_fb *fb)
-{
-	switch (fb->drm_format) {
-	case DRM_FORMAT_RGB565:
-		return 16;
-	case DRM_FORMAT_XRGB8888:
-	case DRM_FORMAT_ARGB8888:
-	case DRM_FORMAT_ARGB2101010:
-	case DRM_FORMAT_XRGB2101010:
-		return 32;
-	default:
-		igt_assert(false);
-	}
-}
-
 static void start_busy_thread(struct igt_fb *fb)
 {
 	int rc;
@@ -1119,7 +1186,7 @@ static void start_busy_thread(struct igt_fb *fb)
 	busy_thread.width = fb->width;
 	busy_thread.height = fb->height;
 	busy_thread.color = pick_color(fb, COLOR_PRIM_BG);
-	busy_thread.bpp = fb_get_bpp(fb);
+	busy_thread.bpp = igt_drm_format_to_bpp(fb->drm_format);
 
 	rc = pthread_create(&busy_thread.thread, NULL, busy_thread_func, NULL);
 	igt_assert_eq(rc, 0);
@@ -1148,17 +1215,17 @@ static void print_crc(const char *str, struct both_crcs *crc)
 	free(pipe_str);
 }
 
-static void collect_crcs(struct both_crcs *crcs)
+static void collect_crcs(struct both_crcs *crcs, bool mandatory_sink_crc)
 {
 	igt_pipe_crc_collect_crc(pipe_crc, &crcs->pipe);
 
 	if (sink_crc.supported)
-		get_sink_crc(&crcs->sink);
+		get_sink_crc(&crcs->sink, mandatory_sink_crc);
 	else
 		memcpy(&crcs->sink, "unsupported!", SINK_CRC_SIZE);
 }
 
-static void init_blue_crc(enum pixel_format format)
+static void init_blue_crc(enum pixel_format format, bool mandatory_sink_crc)
 {
 	struct igt_fb blue;
 	int rc;
@@ -1176,7 +1243,7 @@ static void init_blue_crc(enum pixel_format format)
 			    blue.fb_id, 0, 0, &prim_mode_params.connector_id, 1,
 			    prim_mode_params.mode);
 	igt_assert_eq(rc, 0);
-	collect_crcs(&blue_crcs[format].crc);
+	collect_crcs(&blue_crcs[format].crc, mandatory_sink_crc);
 
 	print_crc("Blue CRC:  ", &blue_crcs[format].crc);
 
@@ -1188,7 +1255,8 @@ static void init_blue_crc(enum pixel_format format)
 }
 
 static void init_crcs(enum pixel_format format,
-		      struct draw_pattern_info *pattern)
+		      struct draw_pattern_info *pattern,
+		      bool mandatory_sink_crc)
 {
 	int r, r_, rc;
 	struct igt_fb tmp_fbs[pattern->n_rects];
@@ -1224,7 +1292,7 @@ static void init_crcs(enum pixel_format format,
 				   &prim_mode_params.connector_id, 1,
 				   prim_mode_params.mode);
 		igt_assert_eq(rc, 0);
-		collect_crcs(&pattern->crcs[format][r]);
+		collect_crcs(&pattern->crcs[format][r], mandatory_sink_crc);
 	}
 
 	for (r = 0; r < pattern->n_rects; r++) {
@@ -1264,10 +1332,14 @@ static void setup_drm(void)
 
 	drm.res = drmModeGetResources(drm.fd);
 	igt_assert(drm.res->count_connectors <= MAX_CONNECTORS);
+	igt_assert(drm.res->count_encoders <= MAX_ENCODERS);
 
 	for (i = 0; i < drm.res->count_connectors; i++)
-		drm.connectors[i] = drmModeGetConnector(drm.fd,
+		drm.connectors[i] = drmModeGetConnectorCurrent(drm.fd,
 						drm.res->connectors[i]);
+	for (i = 0; i < drm.res->count_encoders; i++)
+		drm.encoders[i] = drmModeGetEncoder(drm.fd,
+						    drm.res->encoders[i]);
 
 	rc = drmSetClientCap(drm.fd, DRM_CLIENT_CAP_UNIVERSAL_PLANES, 1);
 	igt_require(rc == 0);
@@ -1295,6 +1367,8 @@ static void teardown_drm(void)
 		drmModeFreePlane(drm.planes[i]);
 	drmModeFreePlaneResources(drm.plane_res);
 
+	for (i = 0; i < drm.res->count_encoders; i++)
+		drmModeFreeEncoder(drm.encoders[i]);
 	for (i = 0; i < drm.res->count_connectors; i++)
 		drmModeFreeConnector(drm.connectors[i]);
 
@@ -1345,18 +1419,21 @@ static void setup_sink_crc(void)
 	errno_ = errno;
 	if (rc == -1 && errno_ == ENOTTY)
 		igt_info("Sink CRC not supported: panel doesn't support it\n");
+	if (rc == -1 && errno_ == ETIMEDOUT)
+		igt_info("Sink CRC not reliable on this panel: skipping it\n");
 	else if (rc == SINK_CRC_SIZE)
 		sink_crc.supported = true;
 	else
-		igt_info("Unexpected sink CRC error, rc=:%ld errno:%d %s\n",
+		igt_info("Unexpected sink CRC error, rc=:%zd errno:%d %s\n",
 			 rc, errno_, strerror(errno_));
 }
 
 static void setup_crcs(void)
 {
 	enum pixel_format f;
+	int crtc_idx = kmstest_get_crtc_idx(drm.res, prim_mode_params.crtc_id);
 
-	pipe_crc = igt_pipe_crc_new(0, INTEL_PIPE_CRC_SOURCE_AUTO);
+	pipe_crc = igt_pipe_crc_new(crtc_idx, INTEL_PIPE_CRC_SOURCE_AUTO);
 
 	setup_sink_crc();
 
@@ -1427,10 +1504,23 @@ static bool fbc_supported_on_chipset(void)
 
 static void setup_fbc(void)
 {
+	drmModeConnectorPtr c = get_connector(prim_mode_params.connector_id);
+
 	if (!fbc_supported_on_chipset()) {
 		igt_info("Can't test FBC: not supported on this chipset\n");
 		return;
 	}
+
+	/*
+	 * While some platforms do allow FBC on pipes B/C, this test suite
+	 * is not prepared for that yet.
+	 * TODO: solve this.
+	 */
+	if (!connector_supports_pipe_a(c)) {
+		igt_info("Can't test FBC: primary connector doesn't support "
+			 "pipe A\n");
+		return;
+	}
 	fbc.can_test = true;
 
 	fbc_setup_last_action();
@@ -1570,19 +1660,23 @@ static int adjust_assertion_flags(const struct test_mode *t, int flags)
 	return flags;
 }
 
-#define do_crc_assertions(flags) do {					\
+#define do_crc_assertions(flags, mandatory_sink_crc) do {		\
 	int flags__ = (flags);						\
 	struct both_crcs crc_;						\
 									\
 	if (!opt.check_crc || (flags__ & DONT_ASSERT_CRC))		\
 		break;							\
 									\
-	collect_crcs(&crc_);						\
+	collect_crcs(&crc_, mandatory_sink_crc);			\
 	print_crc("Calculated CRC:", &crc_);				\
 									\
 	igt_assert(wanted_crc);						\
 	igt_assert_crc_equal(&crc_.pipe, &wanted_crc->pipe);		\
-	assert_sink_crc_equal(&crc_.sink, &wanted_crc->sink);		\
+	if (mandatory_sink_crc)						\
+		assert_sink_crc_equal(&crc_.sink, &wanted_crc->sink);	\
+	else								\
+		if (!sink_crc_equal(&crc_.sink, &wanted_crc->sink))	\
+			igt_info("Sink CRC differ, but not required\n"); \
 } while (0)
 
 #define do_status_assertions(flags_) do {				\
@@ -1618,12 +1712,13 @@ static int adjust_assertion_flags(const struct test_mode *t, int flags)
 
 #define do_assertions(flags) do {					\
 	int flags_ = adjust_assertion_flags(t, (flags));		\
+	bool mandatory_sink_crc = t->feature & FEATURE_PSR;		\
 									\
 	wait_user(2, "Paused before assertions.");			\
 									\
 	/* Check the CRC to make sure the drawing operations work	\
 	 * immediately, independently of the features being enabled. */	\
-	do_crc_assertions(flags_);					\
+	do_crc_assertions(flags_, mandatory_sink_crc);			\
 									\
 	/* Now we can flush things to make the test faster. */		\
 	do_flush(t);							\
@@ -1636,7 +1731,7 @@ static int adjust_assertion_flags(const struct test_mode *t, int flags)
 	 * would only delay the test suite while adding no value to the	\
 	 * test suite. */						\
 	if (t->screen == SCREEN_PRIM)					\
-		do_crc_assertions(flags_);				\
+		do_crc_assertions(flags_, mandatory_sink_crc);		\
 									\
 	if (fbc.supports_last_action && opt.fbc_check_last_action) {	\
 		if (flags_ & ASSERT_LAST_ACTION_CHANGED)		\
@@ -1705,6 +1800,9 @@ static void set_sprite_for_test(const struct test_mode *t,
 
 static void enable_features_for_test(const struct test_mode *t)
 {
+	if (t->feature == FEATURE_DEFAULT)
+		return;
+
 	if (t->feature & FEATURE_FBC)
 		fbc_enable();
 	if (t->feature & FEATURE_PSR)
@@ -1784,7 +1882,7 @@ static void prepare_subtest_data(const struct test_mode *t,
 
 	stop_busy_thread();
 
-	disable_features();
+	disable_features(t);
 	set_crtc_fbs(t);
 
 	if (t->screen == SCREEN_OFFSCREEN)
@@ -1792,9 +1890,9 @@ static void prepare_subtest_data(const struct test_mode *t,
 
 	unset_all_crtcs();
 
-	init_blue_crc(t->format);
+	init_blue_crc(t->format, t->feature & FEATURE_PSR);
 	if (pattern)
-		init_crcs(t->format, pattern);
+		init_crcs(t->format, pattern, t->feature & FEATURE_PSR);
 
 	enable_features_for_test(t);
 }
@@ -1979,6 +2077,7 @@ static void multidraw_subtest(const struct test_mode *t)
 	struct modeset_params *params = pick_params(t);
 	struct fb_region *target;
 	enum igt_draw_method m1, m2, used_method;
+	bool wc_used = false;
 
 	switch (t->plane) {
 	case PLANE_PRI:
@@ -2008,6 +2107,17 @@ static void multidraw_subtest(const struct test_mode *t)
 					igt_draw_get_method_name(used_method));
 
 				draw_rect(pattern, target, used_method, r);
+
+				if (used_method == IGT_DRAW_MMAP_WC)
+					wc_used = true;
+
+				if (used_method == IGT_DRAW_MMAP_GTT &&
+				    wc_used) {
+					struct rect rect =
+						pattern->get_rect(target, r);
+					fb_dirty_ioctl(target, &rect);
+				}
+
 				update_wanted_crc(t,
 						  &pattern->crcs[t->format][r]);
 
@@ -2183,16 +2293,12 @@ static void wait_flip_event(void)
 
 static void set_prim_plane_for_params(struct modeset_params *params)
 {
-	int rc, i, crtc_index = -1;
+	int rc, i;
+	int crtc_idx = kmstest_get_crtc_idx(drm.res, params->crtc_id);
 	uint32_t plane_id = 0;
 
-	for (i = 0; i < drm.res->count_crtcs; i++)
-		if (drm.res->crtcs[i] == params->crtc_id)
-			crtc_index = i;
-	igt_assert(crtc_index >= 0);
-
 	for (i = 0; i < drm.plane_res->count_planes; i++)
-		if ((drm.planes[i]->possible_crtcs & (1 << crtc_index)) &&
+		if ((drm.planes[i]->possible_crtcs & (1 << crtc_idx)) &&
 		    drm.plane_types[i] == DRM_PLANE_TYPE_PRIMARY)
 			plane_id = drm.planes[i]->plane_id;
 	igt_assert(plane_id);
@@ -2489,10 +2595,10 @@ static bool prim_plane_disabled(void)
 {
 	int i, rc;
 	bool disabled, found = false;
+	int crtc_idx = kmstest_get_crtc_idx(drm.res, prim_mode_params.crtc_id);
 
 	for (i = 0; i < drm.plane_res->count_planes; i++) {
-		/* We just pick the first CRTC for the primary plane. */
-		if ((drm.planes[i]->possible_crtcs & 0x1) &&
+		if ((drm.planes[i]->possible_crtcs & (1 << crtc_idx)) &&
 		    drm.plane_types[i] == DRM_PLANE_TYPE_PRIMARY) {
 			found = true;
 			disabled = (drm.planes[i]->crtc_id == 0);
@@ -2600,6 +2706,8 @@ static void scaledprimary_subtest(const struct test_mode *t)
 	struct modeset_params *params = pick_params(t);
 	int i, rc;
 	uint32_t plane_id;
+	int prim_crtc_idx = kmstest_get_crtc_idx(drm.res,
+						 prim_mode_params.crtc_id);
 
 	igt_require_f(intel_gen(intel_get_drm_devid(drm.fd)) >= 9,
 		      "Can't test primary plane scaling before gen 9\n");
@@ -2629,7 +2737,7 @@ static void scaledprimary_subtest(const struct test_mode *t)
 			 pick_color(&new_fb, COLOR_MAGENTA));
 
 	for (i = 0; i < drm.plane_res->count_planes; i++)
-		if ((drm.planes[i]->possible_crtcs & 1) &&
+		if ((drm.planes[i]->possible_crtcs & (1 << prim_crtc_idx)) &&
 		    drm.plane_types[i] == DRM_PLANE_TYPE_PRIMARY)
 			plane_id = drm.planes[i]->plane_id;
 
@@ -2800,9 +2908,10 @@ static void farfromfence_subtest(const struct test_mode *t)
 	struct modeset_params *params = pick_params(t);
 	struct draw_pattern_info *pattern = &pattern1;
 	struct fb_region *target;
-	int max_height;
+	int max_height, assertions = 0;
+	int gen = intel_gen(intel_get_drm_devid(drm.fd));
 
-	switch (intel_gen(intel_get_drm_devid(drm.fd))) {
+	switch (gen) {
 	case 2:
 		max_height = 2048;
 		break;
@@ -2814,6 +2923,11 @@ static void farfromfence_subtest(const struct test_mode *t)
 		break;
 	}
 
+	/* Gen 9 doesn't do the same dspaddr_offset magic as the older
+	 * gens, so FBC may not be enabled there. */
+	if (gen >= 9)
+		assertions |= DONT_ASSERT_FEATURE_STATUS;
+
 	prepare_subtest(t, pattern);
 	target = pick_target(t, params);
 
@@ -2826,13 +2940,14 @@ static void farfromfence_subtest(const struct test_mode *t)
 	params->fb.x = 0;
 	params->fb.y = max_height - params->mode->vdisplay;
 	set_mode_for_params(params);
-	do_assertions(0);
+	do_assertions(assertions);
 
 	for (r = 0; r < pattern->n_rects; r++) {
 		draw_rect(pattern, target, t->method, r);
 		update_wanted_crc(t, &pattern->crcs[t->format][r]);
+
 		/* GTT draws disable PSR. */
-		do_assertions(ASSERT_PSR_DISABLED);
+		do_assertions(assertions | ASSERT_PSR_DISABLED);
 	}
 
 	igt_remove_fb(drm.fd, &tall_fb);
@@ -3032,6 +3147,68 @@ static void tilingchange_subtest(const struct test_mode *t)
 		page_flip_for_params(params, flip_type);
 		do_assertions(0);
 	}
+
+	igt_remove_fb(drm.fd, &new_fb);
+}
+
+/*
+ * basic - do some basic operations regardless of which features are enabled
+ *
+ * METHOD
+ *   This subtest does page flips and draw operations and checks the CRCs of the
+ *   results. The big difference between this and the others is that here we
+ *   don't enable/disable any features such as FBC or PSR: we go with whatever
+ *   the Kernel has enabled by default for us. This subtest only does things
+ *   that are exercised by the other subtests and in a less exhaustive way: it's
+ *   completely redundant. On the other hand, it is very quick and was created
+ *   with the CI system in mind: it's a quick way to detect regressions, so if
+ *   it fails, then we can run the other subtests to find out why.
+ *
+ * EXPECTED RESULTS
+ *   Passed CRC assertions.
+ *
+ * FAILURES
+ *   If you get a failure here, you should run the more specific draw and flip
+ *   subtests of each feature in order to discover what exactly is failing and
+ *   why.
+ *
+ * TODO: do sink CRC assertions in case sink_crc.supported. Only do this after
+ * our sink CRC code gets 100% reliable, in order to avoid CI false negatives.
+ */
+static void basic_subtest(const struct test_mode *t)
+{
+	struct draw_pattern_info *pattern = &pattern1;
+	struct modeset_params *params = pick_params(t);
+	enum igt_draw_method method;
+	struct igt_fb *fb1, fb2;
+	int r;
+	int assertions = DONT_ASSERT_FEATURE_STATUS;
+
+	prepare_subtest(t, pattern);
+
+	create_fb(t->format, params->fb.fb->width, params->fb.fb->height,
+		  LOCAL_I915_FORMAT_MOD_X_TILED, t->plane, &fb2);
+	fb1 = params->fb.fb;
+
+	for (r = 0, method = 0; method < IGT_DRAW_METHOD_COUNT; method++, r++) {
+		if (r == pattern->n_rects) {
+			params->fb.fb = (params->fb.fb == fb1) ? &fb2 : fb1;
+
+			fill_fb_region(&params->fb, COLOR_PRIM_BG);
+			update_wanted_crc(t, &blue_crcs[t->format].crc);
+
+			page_flip_for_params(params, t->flip);
+			do_assertions(assertions);
+
+			r = 0;
+		}
+
+		draw_rect(pattern, &params->fb, method, r);
+		update_wanted_crc(t, &pattern->crcs[t->format][r]);
+		do_assertions(assertions);
+	}
+
+	igt_remove_fb(drm.fd, &fb2);
 }
 
 static int opt_handler(int option, int option_index, void *data)
@@ -3398,7 +3575,7 @@ int main(int argc, char *argv[])
 		if (t.pipes != PIPE_SINGLE ||
 		    t.screen != SCREEN_PRIM ||
 		    t.plane != PLANE_PRI ||
-		    t.method != IGT_DRAW_MMAP_CPU)
+		    t.method != IGT_DRAW_BLT)
 			continue;
 		igt_subtest_f("%s-%s-scaledprimary",
 			      feature_str(t.feature),
@@ -3411,7 +3588,7 @@ int main(int argc, char *argv[])
 		    t.screen != SCREEN_PRIM ||
 		    t.plane != PLANE_PRI ||
 		    t.fbs != FBS_INDIVIDUAL ||
-		    t.method != IGT_DRAW_MMAP_CPU)
+		    t.method != IGT_DRAW_BLT)
 			continue;
 
 		igt_subtest_f("%s-modesetfrombusy", feature_str(t.feature))
@@ -3436,6 +3613,16 @@ int main(int argc, char *argv[])
 			suspend_subtest(&t);
 	TEST_MODE_ITER_END
 
+	t.pipes = PIPE_SINGLE;
+	t.screen = SCREEN_PRIM;
+	t.plane = PLANE_PRI;
+	t.fbs = FBS_INDIVIDUAL;
+	t.feature = FEATURE_DEFAULT;
+	t.format = FORMAT_DEFAULT;
+	t.flip = FLIP_PAGEFLIP;
+	igt_subtest("basic")
+		basic_subtest(&t);
+
 	igt_fixture
 		teardown_environment();
 
