@@ -31,29 +31,6 @@
 #include <unistd.h>
 
 
-/* We know that if we don't enable audio runtime PM, snd_hda_intel will never
- * release its power well refcount, and we'll never reach the LPSP sate. OTOH
- * there's no guarantee that it will release the power well if we enable runtime
- * PM, but at least we can try.  We don't have any assertions since the user may
- * not even have snd_hda_intel loaded, which is not a problem. */
-static void disable_audio_runtime_pm(void)
-{
-	int fd;
-
-	fd = open("/sys/module/snd_hda_intel/parameters/power_save", O_WRONLY);
-	if (fd >= 0) {
-		igt_assert_eq(write(fd, "1\n", 2), 2);
-		close(fd);
-	}
-	fd = open("/sys/bus/pci/devices/0000:00:03.0/power/control", O_WRONLY);
-	if (fd >= 0) {
-		igt_assert_eq(write(fd, "auto\n", 5), 5);
-		close(fd);
-	}
-	/* Give some time for it to react. */
-	sleep(1);
-}
-
 static bool supports_lpsp(uint32_t devid)
 {
 	return IS_HASWELL(devid) || IS_BROADWELL(devid);
@@ -78,16 +55,9 @@ static void screens_disabled_subtest(int drm_fd, drmModeResPtr drm_res)
 static uint32_t create_fb(int drm_fd, int width, int height)
 {
 	struct igt_fb fb;
-	cairo_t *cr;
-	uint32_t buffer_id;
 
-	buffer_id = igt_create_fb(drm_fd, width, height, DRM_FORMAT_XRGB8888,
-				  LOCAL_DRM_FORMAT_MOD_NONE, &fb);
-	cr = igt_get_cairo_ctx(drm_fd, &fb);
-	igt_paint_test_pattern(cr, width, height);
-	cairo_destroy(cr);
-
-	return buffer_id;
+	return igt_create_pattern_fb(drm_fd, width, height, DRM_FORMAT_XRGB8888,
+				     LOCAL_DRM_FORMAT_MOD_NONE, &fb);
 }
 
 static void edp_subtest(int drm_fd, drmModeResPtr drm_res,
@@ -95,7 +65,8 @@ static void edp_subtest(int drm_fd, drmModeResPtr drm_res,
 			bool use_panel_fitter)
 {
 	int i, rc;
-	uint32_t connector_id = 0, crtc_id = 0, buffer_id = 0;
+	uint32_t crtc_id = 0, buffer_id = 0;
+	drmModeConnectorPtr connector = NULL;
 	drmModeModeInfoPtr mode = NULL;
 	drmModeModeInfo std_1024_mode = {
 		.clock = 65000,
@@ -126,12 +97,12 @@ static void edp_subtest(int drm_fd, drmModeResPtr drm_res,
 			continue;
 
 		if (!use_panel_fitter && c->count_modes) {
-			connector_id = c->connector_id;
+			connector = c;
 			mode = &c->modes[0];
 			break;
 		}
 		if (use_panel_fitter) {
-			connector_id = c->connector_id;
+			connector = c;
 
 			/* This is one of the modes Xorg creates for panels, so
 			 * it should work just fine. Notice that Gens that
@@ -146,18 +117,18 @@ static void edp_subtest(int drm_fd, drmModeResPtr drm_res,
 			break;
 		}
 	}
-	igt_require(connector_id);
+	igt_require(connector);
 
-	crtc_id = drm_res->crtcs[0];
+	crtc_id = kmstest_find_crtc_for_connector(drm_fd, drm_res, connector,
+						  0);
 	buffer_id = create_fb(drm_fd, mode->hdisplay, mode->vdisplay);
 
-	igt_assert(crtc_id);
 	igt_assert(buffer_id);
-	igt_assert(connector_id);
+	igt_assert(connector);
 	igt_assert(mode);
 
-	rc = drmModeSetCrtc(drm_fd, crtc_id, buffer_id, 0, 0, &connector_id, 1,
-			    mode);
+	rc = drmModeSetCrtc(drm_fd, crtc_id, buffer_id, 0, 0,
+			    &connector->connector_id, 1, mode);
 	igt_assert_eq(rc, 0);
 
 	if (use_panel_fitter) {
@@ -174,7 +145,8 @@ static void non_edp_subtest(int drm_fd, drmModeResPtr drm_res,
 			    drmModeConnectorPtr *drm_connectors)
 {
 	int i, rc;
-	uint32_t connector_id = 0, crtc_id = 0, buffer_id = 0;
+	uint32_t crtc_id = 0, buffer_id = 0;
+	drmModeConnectorPtr connector = NULL;
 	drmModeModeInfoPtr mode = NULL;
 
 	kmstest_unset_all_crtcs(drm_fd, drm_res);
@@ -188,23 +160,22 @@ static void non_edp_subtest(int drm_fd, drmModeResPtr drm_res,
 			continue;
 
 		if (c->count_modes) {
-			connector_id = c->connector_id;
+			connector = c;
 			mode = &c->modes[0];
 			break;
 		}
 	}
-	igt_require(connector_id);
+	igt_require(connector);
 
-	crtc_id = drm_res->crtcs[0];
+	crtc_id = kmstest_find_crtc_for_connector(drm_fd, drm_res, connector,
+						  0);
 	buffer_id = create_fb(drm_fd, mode->hdisplay, mode->vdisplay);
 
-	igt_assert(crtc_id);
 	igt_assert(buffer_id);
-	igt_assert(connector_id);
 	igt_assert(mode);
 
-	rc = drmModeSetCrtc(drm_fd, crtc_id, buffer_id, 0, 0, &connector_id, 1,
-			    mode);
+	rc = drmModeSetCrtc(drm_fd, crtc_id, buffer_id, 0, 0,
+			    &connector->connector_id, 1, mode);
 	igt_assert_eq(rc, 0);
 
 	igt_assert(!lpsp_is_enabled(drm_fd));
@@ -231,10 +202,10 @@ igt_main
 		igt_assert(drm_res->count_connectors <= MAX_CONNECTORS);
 
 		for (i = 0; i < drm_res->count_connectors; i++)
-			drm_connectors[i] = drmModeGetConnector(drm_fd,
+			drm_connectors[i] = drmModeGetConnectorCurrent(drm_fd,
 							drm_res->connectors[i]);
 
-		disable_audio_runtime_pm();
+		igt_pm_enable_audio_runtime_pm();
 
 		igt_require(supports_lpsp(devid));
 
